# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2013 credativ (http://www.credativ.co.uk). All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
#from openerp.tools.translate import _
import redmine_osv

import logging
_logger = logging.getLogger(__name__)
_logger.setLevel(logging.DEBUG)

DEBUG = True

class external_referential(redmine_osv.redmine_osv):
    '''
    Add methods to import/export projects, tasks, and timesheet
    entries from/to Redmine.
    '''
    _inherit = 'external.referential'

    _columns = {
        'active': fields.boolean(
            'Active'),
    }

    _defaults = {
        'active': 1,
    }

    def external_connection(self, cr, uid, ids, DEBUG=False, context=None):
        if isinstance(ids, (tuple, list)):
            id = ids[0]
        else:
            id = ids
        referential = self.browse(cr, uid, id, context=context)

        if 'redmine' in referential.type_id.name.lower():
            attr_conn = redmine_osv.Connection(referential.location, referential.apiusername, referential.apipass, DEBUG)
            return attr_conn.connect() and attr_conn or False
        else:
            return super(external_referential, self).external_connection(cr, uid, id, DEBUG=DEBUG, context=context)

    def import_projects(self, cr, uid, ids, context=None):
        raise NotImplementedError('Projects import is not to be used yet.')

        for referential in self.browse(cr, uid, ids, context=context):
            conn = self.external_connection(cr, uid, referential.id, DEBUG, context=context)
            if conn:
                self.pool.get('project.project').rm_import_base(cr, uid, conn, referential.id, defaults={'referential_id': referential.id})
        return True

    def export_projects(self, cr, uid, ids, context=None):
        raise NotImplementedError('Projects export is not to be used yet.')

        project_pool = self.pool.get('project.project')
        project_ids = project_pool.search(cr, uid, [], context=context)
        for referential in self.browse(cr, uid, ids, context=context):
            if not referential.active:
                continue
            for project in project_pool.read(cr, uid, project_ids, [], context=context):
                project_pool.ext_export_one(cr, uid, redmine_osv.clean_read_m2o(project), referential.id, context=context)
        return True

    def import_tasks(self, cr, uid, ids, context=None):
        for referential in self.browse(cr, uid, ids, context=context):
            if not referential.active:
                continue
            conn = self.external_connection(cr, uid, referential.id, DEBUG, context=context)
            if conn:
                self.pool.get('project.task').rm_import_base(cr, uid, conn, referential.id, defaults={'referential_id': referential.id})
        return True

    def export_tasks(self, cr, uid, ids, context=None):
        task_pool = self.pool.get('project.task')
        task_ids = task_pool.search(cr, uid, [], context=context)
        for referential in self.browse(cr, uid, ids, context=context):
            if not referential.active:
                continue
            for task in task_pool.read(cr, uid, task_ids, [], context=context):
                task_pool.ext_export_one(cr, uid, redmine_osv.clean_read_m2o(task), referential.id, context=context)
        return True

    def import_works(self, cr, uid, ids, context=None):
        for referential in self.browse(cr, uid, ids, context=context):
            if not referential.active:
                continue
            conn = self.external_connection(cr, uid, referential.id, DEBUG, context=context)
            if conn:
                self.pool.get('project.task.work').rm_import_base(cr, uid, conn, referential.id, defaults={'referential_id': referential.id})
        return True

    def export_works(self, cr, uid, ids, context=None):
        work_pool = self.pool.get('project.task.work')
        work_ids = work_pool.search(cr, uid, [], context=context)
        for referential in self.browse(cr, uid, ids, context=context):
            if not referential.active:
                continue
            for work in work_pool.read(cr, uid, work_ids, [], context=context):
                work_pool.ext_export_one(cr, uid, redmine_osv.clean_read_m2o(work), referential.id, context=context)
        return True

    # def import_timesheet_entries(self, cr, uid, ids, context=None):
    #     for referential in self.browse(cr, uid, ids, context=context):
    #         if not referential.active:
    #             continue
    #         conn = self.external_connection(cr, uid, referential.id, DEBUG, context=context)
    #         if conn:
    #             self.pool.get('hr.analytic.timesheet').rm_import_base(cr, uid, conn, referential.id, defaults={'referential_id': referential.id})
    #     return True

    # def export_timesheet_entries(self, cr, uid, ids, context=None):
    #     timesheet_pool = self.pool.get('hr.analytic.timesheet')
    #     timesheet_ids = timesheet_pool.search(cr, uid, [], context=context)
    #     for referential in self.browse(cr, uid, ids, context=context):
    #         if not referential.active:
    #             continue
    #         for timesheet in timesheet_pool.read(cr, uid, timesheet_ids, [], context=context):
    #             timesheet_pool.ext_export_one(cr, uid, redmine_osv.clean_read_m2o(timesheet), referential.id, context=context)
    #     return True

external_referential()


class external_mapping(osv.osv):
    '''
    Add some default settings for Redmine mappings.
    '''
    _inherit = 'external.mapping'

    def _default_rm_list_method(self, cr, uid, context={}):
        return '%s.find' % (context.get('rm_model', ''),)

    def _default_rm_update_method(self, cr, uid, context={}):
        return '%s.put' % (context.get('rm_model', ''),)

    def _default_rm_delete_method(self, cr, uid, context={}):
        return '%s.destroy' % (context.get('rm_model', ''),)

    def _default_rm_get_method(self, cr, uid, context={}):
        return '%s.find_one' % (context.get('rm_model', ''),)

    def _default_rm_create_method(self, cr, uid, context={}):
        return '%s.create' % (context.get('rm_model', ''),)

    _defaults = {
        'external_list_method': _default_rm_list_method,
        'external_update_method': _default_rm_update_method,
        'external_delete_method': _default_rm_delete_method,
        'external_get_method': _default_rm_get_method,
        'external_create_method': _default_rm_create_method,
    }

external_mapping()
