# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2013 credativ (http://www.credativ.co.uk). All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
from redmine_osv import Connection

import logging
_logger = logging.getLogger(__name__)
_logger.setLevel(logging.DEBUG)

class users(osv.osv):
    '''
    Add a redmine login name field to res.users.
    '''
    _inherit = 'res.users'

    def _get_rm_user_id(self, cr, uid, ids, field, args, context=None):
        referential_type_pool = self.pool.get('external.referential.type')
        type_ids = referential_type_pool.search(cr, uid, [('name','ilike','redmine')], context=context)
        
        referential_pool = self.pool.get('external.referential')
        referential_ids = referential_pool.search(cr, uid, [('type_id','in',type_ids)], context=context)

        if len(referential_ids) == 1:
            referential = referential_pool.browse(cr, uid, referential_ids[0], context=context)
            try:
                conn = Connection(referential.location, referential.apiusername, referential.apipass)
                return dict([(user.id, conn.call('Users.find', user.rm_login))
                             for user in self.browse(cr, uid, ids, context=context)])
            except Exception, e:
                _logger.error('Redmine: could not retrieve redmine user ID(s): %s' % (str(e),))
        elif len(referential_ids) > 0:
            raise osv.except_osv('Integrity error',
                                 'Found multiple external referentials of type Redmine')
        else:
            raise osv.except_osv('Integrity error',
                                 'Could not find an external referential of type Redmine')

    def _update_rm_user_id(self, cr, uid, ids, context=None):
        return self._get_rm_user_id(cr, uid, ids, field='rm_user_id', args=[], context=context)

    _columns = {
        'rm_login': fields.char(
            'Redmine login',
            size=64,
            help='This user\'s login name on Redmine'),
        # FIXME Seems like access to users.xml is further restricted somehow
        # 'rm_user_id': fields.function(
        #     _get_rm_user_id,
        #     store={'res.users': (_update_rm_user_id, ['rm_login'], 10)},
        #     type='integer',
        #     string='Redmine user ID',
        #     readonly=True),
        # FIXME For now we'll just required that the user ID is manually entered
        'rm_user_id': fields.integer(
            string='Redmine user ID'),
    }

    _defaults = {
        'rm_user_id': -1,
    }

users()
